@@ -2,6 +2,7 @@ package clusterprovision
 
 import (
 	"context"
+	"fmt"
 	"time"
 
 	"github.com/pkg/errors"
@@ -15,6 +16,8 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
+	"k8s.io/client-go/util/flowcontrol"
+	"k8s.io/client-go/util/workqueue"
 
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"sigs.k8s.io/controller-runtime/pkg/controller"
@@ -33,7 +36,7 @@ import (
 )
 
 const (
-	ControllerName = "clusterProvision"
+	ControllerName = hivev1.ClusterProvisionControllerName
 
 	// clusterProvisionLabelKey is the label that is used to identify
 	// resources descendant from a cluster provision.
@@ -41,6 +44,8 @@ const (
 
 	resultSuccess = "success"
 	resultFailure = "failure"
+
+	podStatusCheckDelay = 60 * time.Second
 )
 
 var (
@@ -69,14 +74,20 @@ func init() {
 // Add creates a new ClusterProvision Controller and adds it to the Manager with default RBAC. The Manager will set fields on the Controller
 // and Start it when the Manager is Started.
 func Add(mgr manager.Manager) error {
-	return add(mgr, newReconciler(mgr))
+	logger := log.WithField("controller", ControllerName)
+	concurrentReconciles, clientRateLimiter, queueRateLimiter, err := controllerutils.GetControllerConfig(mgr.GetClient(), ControllerName)
+	if err != nil {
+		logger.WithError(err).Error("could not get controller configurations")
+		return err
+	}
+	return add(mgr, newReconciler(mgr, clientRateLimiter), concurrentReconciles, queueRateLimiter)
 }
 
 // newReconciler returns a new reconcile.Reconciler
-func newReconciler(mgr manager.Manager) reconcile.Reconciler {
+func newReconciler(mgr manager.Manager, rateLimiter flowcontrol.RateLimiter) reconcile.Reconciler {
 	logger := log.WithField("controller", ControllerName)
 	return &ReconcileClusterProvision{
-		Client:       controllerutils.NewClientWithMetricsOrDie(mgr, ControllerName),
+		Client:       controllerutils.NewClientWithMetricsOrDie(mgr, ControllerName, &rateLimiter),
 		scheme:       mgr.GetScheme(),
 		logger:       logger,
 		expectations: controllerutils.NewExpectations(logger),
@@ -84,14 +95,18 @@ func newReconciler(mgr manager.Manager) reconcile.Reconciler {
 }
 
 // add adds a new Controller to mgr with r as the reconcile.Reconciler
-func add(mgr manager.Manager, r reconcile.Reconciler) error {
+func add(mgr manager.Manager, r reconcile.Reconciler, concurrentReconciles int, rateLimiter workqueue.RateLimiter) error {
 	provisionReconciler, ok := r.(*ReconcileClusterProvision)
 	if !ok {
 		return errors.New("reconciler supplied is not a ReconcileClusterProvision")
 	}
 
 	// Create a new controller
-	c, err := controller.New("clusterprovision-controller", mgr, controller.Options{Reconciler: r})
+	c, err := controller.New("clusterprovision-controller", mgr, controller.Options{
+		Reconciler:              r,
+		MaxConcurrentReconciles: concurrentReconciles,
+		RateLimiter:             rateLimiter,
+	})
 	if err != nil {
 		return errors.Wrap(err, "could not create controller")
 	}
@@ -137,7 +152,7 @@ func (r *ReconcileClusterProvision) Reconcile(request reconcile.Request) (reconc
 	pLog.Info("reconciling cluster provision")
 	defer func() {
 		dur := time.Since(start)
-		hivemetrics.MetricControllerReconcileTime.WithLabelValues(ControllerName).Observe(dur.Seconds())
+		hivemetrics.MetricControllerReconcileTime.WithLabelValues(ControllerName.String()).Observe(dur.Seconds())
 		pLog.WithField("elapsed", dur).Info("reconcile complete")
 	}()
 
@@ -246,7 +261,7 @@ func (r *ReconcileClusterProvision) createJob(instance *hivev1.ClusterProvision,
 
 func (r *ReconcileClusterProvision) adoptJob(instance *hivev1.ClusterProvision, job *batchv1.Job, pLog log.FieldLogger) (reconcile.Result, error) {
 	instance.Status.JobRef = &corev1.LocalObjectReference{Name: job.Name}
-	return reconcile.Result{}, r.setCondition(instance, hivev1.ClusterProvisionJobCreated, "JobCreated", "Install job has been created", pLog)
+	return reconcile.Result{}, r.setCondition(instance, hivev1.ClusterProvisionJobCreated, corev1.ConditionTrue, "JobCreated", "Install job has been created", controllerutils.UpdateConditionAlways, pLog)
 }
 
 // check if the job has completed
@@ -258,7 +273,7 @@ func (r *ReconcileClusterProvision) reconcileRunningJob(instance *hivev1.Cluster
 	case apierrors.IsNotFound(err):
 		if cond := controllerutils.FindClusterProvisionCondition(instance.Status.Conditions, hivev1.ClusterProvisionFailedCondition); cond == nil {
 			pLog.Error("install job lost")
-			if err := r.setCondition(instance, hivev1.ClusterProvisionFailedCondition, "JobNotFound", "install job not found", pLog); err != nil {
+			if err := r.setCondition(instance, hivev1.ClusterProvisionFailedCondition, corev1.ConditionTrue, "JobNotFound", "install job not found", controllerutils.UpdateConditionAlways, pLog); err != nil {
 				return reconcile.Result{}, err
 			}
 		} else {
@@ -285,6 +300,33 @@ func (r *ReconcileClusterProvision) reconcileRunningJob(instance *hivev1.Cluster
 
 	pLog.Debug("install job still running")
 
+	if time.Since(job.CreationTimestamp.Time) > podStatusCheckDelay {
+		installPod, err := r.getInstallPod(job, pLog)
+		if err != nil {
+			pLog.WithError(err).Error("could not get install pod")
+			if err := r.setCondition(instance, hivev1.InstallPodStuckCondition, corev1.ConditionTrue, "InstallPodMissing", err.Error(), controllerutils.UpdateConditionIfReasonOrMessageChange, pLog); err != nil {
+				return reconcile.Result{}, err
+			}
+			return reconcile.Result{}, err
+		}
+
+		if installPod.Status.Phase == "Pending" {
+			pLog.WithField("pod", installPod.Name).Error("install pod is stuck")
+			if err := r.setCondition(instance, hivev1.InstallPodStuckCondition, corev1.ConditionTrue, "PodInPendingPhase", "pod is in pending phase", controllerutils.UpdateConditionIfReasonOrMessageChange, pLog); err != nil {
+				return reconcile.Result{}, err
+			}
+			// Since this controller is not watching pods, the ClusterProvision will not be re-synced if the pod does
+			// transition to the running phase later. However, if the pod does start running, then soon after either the
+			// install manager will set the InfraID on the ClusterProvision or the pod will fail.
+			return reconcile.Result{}, nil
+		}
+		if cond := controllerutils.FindClusterProvisionCondition(instance.Status.Conditions, hivev1.InstallPodStuckCondition); cond != nil && cond.Status == corev1.ConditionTrue {
+			if err := r.setCondition(instance, hivev1.InstallPodStuckCondition, corev1.ConditionFalse, "PodInRunningPhase", "pod is in running phase", controllerutils.UpdateConditionNever, pLog); err != nil {
+				return reconcile.Result{}, err
+			}
+		}
+	}
+
 	if instance.Spec.Stage == hivev1.ClusterProvisionStageInitializing && instance.Spec.InfraID != nil {
 		cd := hivev1.ClusterDeployment{}
 		if err := r.Get(
@@ -303,9 +345,41 @@ func (r *ReconcileClusterProvision) reconcileRunningJob(instance *hivev1.Cluster
 		}
 	}
 
+	if timeUntilNextPodStatusCheck := podStatusCheckDelay - time.Since(job.CreationTimestamp.Time); timeUntilNextPodStatusCheck > 0 {
+		return reconcile.Result{RequeueAfter: timeUntilNextPodStatusCheck}, nil
+	}
 	return reconcile.Result{}, nil
 }
 
+func (r *ReconcileClusterProvision) getInstallPod(job *batchv1.Job, pLog log.FieldLogger) (*corev1.Pod, error) {
+	podList := &corev1.PodList{}
+	podLabelSelector, err := metav1.LabelSelectorAsSelector(job.Spec.Selector)
+	if err != nil {
+		pLog.WithError(err).Error("could not create pod selector from job")
+		return nil, fmt.Errorf("could not create pod selector from job")
+	}
+	if err := r.List(
+		context.TODO(),
+		podList,
+		client.MatchingLabelsSelector{Selector: podLabelSelector},
+		client.InNamespace(job.Namespace),
+	); err != nil {
+		pLog.WithError(err).Error("could not list install pods")
+		return nil, fmt.Errorf("could not list install pods")
+	}
+
+	switch len(podList.Items) {
+	case 0:
+		pLog.Error("install pod not found")
+		return nil, fmt.Errorf("install pod not found")
+	case 1:
+		return &podList.Items[0], nil
+	default:
+		pLog.Error("more than one install pod exists")
+		return nil, fmt.Errorf("more than one install pod exists")
+	}
+}
+
 func (r *ReconcileClusterProvision) reconcileSuccessfulJob(instance *hivev1.ClusterProvision, job *batchv1.Job, pLog log.FieldLogger) (reconcile.Result, error) {
 	pLog.Info("install job succeeded")
 	result, err := r.transitionStage(instance, hivev1.ClusterProvisionStageComplete, "InstallComplete", "Install job has completed successfully", pLog)
@@ -337,7 +411,7 @@ func (r *ReconcileClusterProvision) abortProvision(instance *hivev1.ClusterProvi
 	if instance.Status.JobRef == nil {
 		return r.transitionStage(instance, hivev1.ClusterProvisionStageFailed, reason, message, pLog)
 	}
-	if err := r.setCondition(instance, hivev1.ClusterProvisionFailedCondition, reason, message, pLog); err != nil {
+	if err := r.setCondition(instance, hivev1.ClusterProvisionFailedCondition, corev1.ConditionTrue, reason, message, controllerutils.UpdateConditionAlways, pLog); err != nil {
 		return reconcile.Result{}, err
 	}
 	job := &batchv1.Job{}
@@ -377,7 +451,7 @@ func (r *ReconcileClusterProvision) transitionStage(
 	default:
 		return reconcile.Result{}, errors.New("unknown stage")
 	}
-	if err := r.setCondition(instance, conditionType, reason, message, pLog); err != nil {
+	if err := r.setCondition(instance, conditionType, corev1.ConditionTrue, reason, message, controllerutils.UpdateConditionAlways, pLog); err != nil {
 		return reconcile.Result{}, err
 	}
 	if err := r.setStage(instance, stage, pLog); err != nil {
@@ -389,17 +463,19 @@ func (r *ReconcileClusterProvision) transitionStage(
 func (r *ReconcileClusterProvision) setCondition(
 	instance *hivev1.ClusterProvision,
 	conditionType hivev1.ClusterProvisionConditionType,
+	status corev1.ConditionStatus,
 	reason string,
 	message string,
+	updateConditionCheck controllerutils.UpdateConditionCheck,
 	pLog log.FieldLogger,
 ) error {
 	instance.Status.Conditions = controllerutils.SetClusterProvisionCondition(
 		instance.Status.Conditions,
 		conditionType,
-		corev1.ConditionTrue,
+		status,
 		reason,
 		message,
-		controllerutils.UpdateConditionAlways,
+		updateConditionCheck,
 	)
 	if err := r.Status().Update(context.TODO(), instance); err != nil {
 		pLog.WithError(err).Error("cannot update status conditions")
@@ -465,6 +541,7 @@ func generateOwnershipUniqueKeys(owner hivev1.MetaRuntimeObject) []*controllerut
 				constants.ClusterProvisionNameLabel: owner.GetName(),
 				constants.JobTypeLabel:              constants.JobTypeProvision,
 			},
+			Controlled: true,
 		},
 		{
 			TypeToList: &corev1.SecretList{},
@@ -472,6 +549,7 @@ func generateOwnershipUniqueKeys(owner hivev1.MetaRuntimeObject) []*controllerut
 				constants.ClusterProvisionNameLabel: owner.GetName(),
 				constants.SecretTypeLabel:           constants.SecretTypeKubeConfig,
 			},
+			Controlled: false,
 		},
 		{
 			TypeToList: &corev1.SecretList{},
@@ -479,6 +557,7 @@ func generateOwnershipUniqueKeys(owner hivev1.MetaRuntimeObject) []*controllerut
 				constants.ClusterProvisionNameLabel: owner.GetName(),
 				constants.SecretTypeLabel:           constants.SecretTypeKubeAdminCreds,
 			},
+			Controlled: false,
 		},
 	}
 }
